Package: RELIEF
Type: Package
Title: RELIEF harmonization for removing latent patch effects
Version: 0.1.0
Author: Jun Young Park, Rongqian Zhang
Maintainer: Jun Young Park <junjy.park@utronto.ca>
Description: TBD
License: GPL-3
Encoding: UTF-8
Depends: Matrix, MASS, denoiseR
 